Package: makeit
Version: 1.0.2
Date: 2025-03-18
Title: Run R Scripts if Needed
Authors@R: c(person("Arni", "Magnusson", role=c("aut","cre"), email="thisisarni@gmail.com"))
Suggests: dplyr, ggplot2, knitr, readr, rmarkdown
Description: Automation tool to run R scripts if needed, based on last modified
  time. It comes with no package dependencies, organizational overhead, or
  structural requirements. In short: run an R script if underlying files have
  changed, otherwise do nothing.
License: GPL-3
URL: https://github.com/arni-magnusson/makeit
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-03-17 22:02:44 UTC; arnim
Author: Arni Magnusson [aut, cre]
Maintainer: Arni Magnusson <thisisarni@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-17 22:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 20:09:56 UTC; unix
